from google.protobuf import empty_pb2 as _empty_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import (
    ClassVar as _ClassVar,
    Iterable as _Iterable,
    Mapping as _Mapping,
    Optional as _Optional,
    Union as _Union,
)

DESCRIPTOR: _descriptor.FileDescriptor

class Status(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    SUCCESS: _ClassVar[Status]
    FAILURE: _ClassVar[Status]
    FALLBACK: _ClassVar[Status]

SUCCESS: Status
FAILURE: Status
FALLBACK: Status

class SinkRequest(_message.Message):
    __slots__ = ("request", "status", "handshake")

    class Request(_message.Message):
        __slots__ = ("keys", "value", "event_time", "watermark", "id", "headers")

        class HeadersEntry(_message.Message):
            __slots__ = ("key", "value")
            KEY_FIELD_NUMBER: _ClassVar[int]
            VALUE_FIELD_NUMBER: _ClassVar[int]
            key: str
            value: str
            def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
        KEYS_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        EVENT_TIME_FIELD_NUMBER: _ClassVar[int]
        WATERMARK_FIELD_NUMBER: _ClassVar[int]
        ID_FIELD_NUMBER: _ClassVar[int]
        HEADERS_FIELD_NUMBER: _ClassVar[int]
        keys: _containers.RepeatedScalarFieldContainer[str]
        value: bytes
        event_time: _timestamp_pb2.Timestamp
        watermark: _timestamp_pb2.Timestamp
        id: str
        headers: _containers.ScalarMap[str, str]
        def __init__(
            self,
            keys: _Optional[_Iterable[str]] = ...,
            value: _Optional[bytes] = ...,
            event_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
            watermark: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
            id: _Optional[str] = ...,
            headers: _Optional[_Mapping[str, str]] = ...,
        ) -> None: ...
    REQUEST_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    HANDSHAKE_FIELD_NUMBER: _ClassVar[int]
    request: SinkRequest.Request
    status: TransmissionStatus
    handshake: Handshake
    def __init__(
        self,
        request: _Optional[_Union[SinkRequest.Request, _Mapping]] = ...,
        status: _Optional[_Union[TransmissionStatus, _Mapping]] = ...,
        handshake: _Optional[_Union[Handshake, _Mapping]] = ...,
    ) -> None: ...

class Handshake(_message.Message):
    __slots__ = ("sot",)
    SOT_FIELD_NUMBER: _ClassVar[int]
    sot: bool
    def __init__(self, sot: bool = ...) -> None: ...

class ReadyResponse(_message.Message):
    __slots__ = ("ready",)
    READY_FIELD_NUMBER: _ClassVar[int]
    ready: bool
    def __init__(self, ready: bool = ...) -> None: ...

class TransmissionStatus(_message.Message):
    __slots__ = ("eot",)
    EOT_FIELD_NUMBER: _ClassVar[int]
    eot: bool
    def __init__(self, eot: bool = ...) -> None: ...

class SinkResponse(_message.Message):
    __slots__ = ("results", "handshake", "status")

    class Result(_message.Message):
        __slots__ = ("id", "status", "err_msg")
        ID_FIELD_NUMBER: _ClassVar[int]
        STATUS_FIELD_NUMBER: _ClassVar[int]
        ERR_MSG_FIELD_NUMBER: _ClassVar[int]
        id: str
        status: Status
        err_msg: str
        def __init__(
            self,
            id: _Optional[str] = ...,
            status: _Optional[_Union[Status, str]] = ...,
            err_msg: _Optional[str] = ...,
        ) -> None: ...
    RESULTS_FIELD_NUMBER: _ClassVar[int]
    HANDSHAKE_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    results: _containers.RepeatedCompositeFieldContainer[SinkResponse.Result]
    handshake: Handshake
    status: TransmissionStatus
    def __init__(
        self,
        results: _Optional[_Iterable[_Union[SinkResponse.Result, _Mapping]]] = ...,
        handshake: _Optional[_Union[Handshake, _Mapping]] = ...,
        status: _Optional[_Union[TransmissionStatus, _Mapping]] = ...,
    ) -> None: ...



from collections.abc import AsyncIterable

from google.protobuf import empty_pb2 as _empty_pb2

from pynumaflow.mapstreamer import Datum
from pynumaflow.mapstreamer._dtypes import MapStreamCallable, MapStreamError
from pynumaflow.proto.mapper import map_pb2_grpc, map_pb2
from pynumaflow.shared.server import exit_on_error
from pynumaflow.types import NumaflowServicerContext
from pynumaflow._constants import _LOGGER


class AsyncMapStreamServicer(map_pb2_grpc.MapServicer):
    """
    This class is used to create a new grpc Map Stream Servicer instance.
    It implements the SyncMapServicer interface from the proto
    map_pb2_grpc.py file.
    Provides the functionality for the required rpc methods.
    """

    def __init__(
        self,
        handler: MapStreamCallable,
    ):
        self.__map_stream_handler: MapStreamCallable = handler

    async def MapFn(
        self,
        request_iterator: AsyncIterable[map_pb2.MapRequest],
        context: NumaflowServicerContext,
    ) -> AsyncIterable[map_pb2.MapResponse]:
        """
        Applies a map function to a datum stream in streaming mode.
        The pascal case function name comes from the proto map_pb2_grpc.py file.
        """
        try:
            # The first message to be received should be a valid handshake
            req = await request_iterator.__anext__()
            # check if it is a valid handshake req
            if not (req.handshake and req.handshake.sot):
                raise MapStreamError("MapStreamFn: expected handshake as the first message")
            yield map_pb2.MapResponse(handshake=map_pb2.Handshake(sot=True))

            # read for each input request
            async for req in request_iterator:
                # yield messages as received from the UDF
                async for res in self.__invoke_map_stream(
                    list(req.request.keys),
                    Datum(
                        keys=list(req.request.keys),
                        value=req.request.value,
                        event_time=req.request.event_time.ToDatetime(),
                        watermark=req.request.watermark.ToDatetime(),
                        headers=dict(req.request.headers),
                    ),
                ):
                    yield map_pb2.MapResponse(results=[res], id=req.id)
                # send EOT to indicate end of transmission for a given message
                yield map_pb2.MapResponse(status=map_pb2.TransmissionStatus(eot=True), id=req.id)
        except BaseException as err:
            _LOGGER.critical("UDFError, re-raising the error", exc_info=True)
            exit_on_error(context, repr(err))
            return

    async def __invoke_map_stream(self, keys: list[str], req: Datum):
        try:
            # Invoke the user handler for map stream
            async for msg in self.__map_stream_handler(keys, req):
                yield map_pb2.MapResponse.Result(keys=msg.keys, value=msg.value, tags=msg.tags)
        except BaseException as err:
            _LOGGER.critical("UDFError, re-raising the error", exc_info=True)
            raise err

    async def IsReady(
        self, request: _empty_pb2.Empty, context: NumaflowServicerContext
    ) -> map_pb2.ReadyResponse:
        """
        IsReady is the heartbeat endpoint for gRPC.
        The pascal case function name comes from the proto map_pb2_grpc.py file.
        """
        return map_pb2.ReadyResponse(ready=True)



import os
from abc import ABCMeta, abstractmethod
from collections.abc import Iterable
from dataclasses import dataclass
from datetime import datetime
from typing import Callable, Optional
from collections.abc import AsyncIterable

from pynumaflow.shared.asynciter import NonBlockingIterator


@dataclass(init=False)
class Offset:
    """
    Args:
        offset: the offset of the datum.
        partition_id: partition_id indicates which partition of the source the datum belongs to.
    """

    __slots__ = ("_offset", "_partition_id")

    _offset: bytes
    _partition_id: int

    def __init__(self, offset: bytes, partition_id: int):
        self._offset = offset
        self._partition_id = partition_id

    @classmethod
    def offset_with_default_partition_id(cls, offset: bytes):
        """
        Returns an Offset object with the given offset and default partition id.
        """
        return Offset(offset=offset, partition_id=get_default_partitions()[0])

    @property
    def as_dict(self):
        return {"offset": self._offset, "partition_id": self._partition_id}

    @property
    def offset(self) -> bytes:
        return self._offset

    @property
    def partition_id(self) -> int:
        return self._partition_id


@dataclass(init=False)
class Message:
    """
    Basic datatype for data passing to the next vertex/vertices.

    Args:
        payload: data in bytes
        offset: the offset of the datum.
        event_time: event time of the message, usually extracted from the payload.
        keys: []string keys for vertex (optional)
        headers: dict of headers for the message (optional)
    """

    __slots__ = ("_payload", "_offset", "_event_time", "_keys", "_headers")

    _payload: bytes
    _offset: Offset
    _event_time: datetime
    _keys: list[str]
    _headers: dict[str, str]

    def __init__(
        self,
        payload: bytes,
        offset: Offset,
        event_time: datetime,
        keys: list[str] = None,
        headers: Optional[dict[str, str]] = None,
    ):
        """
        Creates a Message object to send value to a vertex.
        """
        self._payload = payload
        self._offset = offset
        self._event_time = event_time
        self._keys = keys or []
        self._headers = headers or {}

    @property
    def payload(self) -> bytes:
        return self._payload

    @property
    def keys(self) -> list[str]:
        return self._keys

    @property
    def offset(self) -> Offset:
        return self._offset

    @property
    def event_time(self) -> datetime:
        return self._event_time

    @property
    def headers(self) -> dict[str, str]:
        return self._headers


@dataclass(init=False)
class ReadRequest:
    """
    Class to define the request for reading datum stream from user defined source.
    Args:
        num_records: the number of records to read.
        timeout_in_ms: the request timeout in milliseconds.
    >>> # Example usage
    >>> from pynumaflow.sourcer import ReadRequest
    >>> read_request = ReadRequest(num_records=10, timeout_in_ms=1000)
    """

    __slots__ = ("_num_records", "_timeout_in_ms")

    _num_records: int
    _timeout_in_ms: int

    def __init__(
        self,
        num_records: int,
        timeout_in_ms: int,
    ):
        if not isinstance(num_records, int):
            raise TypeError(f"Wrong data type: {type(num_records)} for ReadRequest.num_records")
        self._num_records = num_records
        if not isinstance(timeout_in_ms, int):
            raise TypeError(f"Wrong data type: {type(timeout_in_ms)} for ReadRequest.timeout_in_ms")
        self._timeout_in_ms = timeout_in_ms

    @property
    def num_records(self) -> int:
        """Returns the num_records of the request"""
        return self._num_records

    @property
    def timeout_in_ms(self) -> int:
        """Returns the timeout_in_ms of the request."""
        return self._timeout_in_ms


@dataclass(init=False)
class AckRequest:
    """
    Class for defining the request for acknowledging datum.
    It takes a list of offsets that need to be acknowledged.
    Args:
        offsets: the offsets to be acknowledged.
    >>> # Example usage
    >>> from pynumaflow.sourcer import AckRequest, Offset
    >>> offset_val = Offset(offset=b"123", partition_id=0)
    >>> ack_request = AckRequest(offsets=[offset_val, offset_val])
    """

    __slots__ = ("_offsets",)
    _offsets: list[Offset]

    def __init__(self, offsets: list[Offset]):
        self._offsets = offsets

    @property
    def offsets(self) -> list[Offset]:
        """Returns the offsets to be acknowledged."""
        return self._offsets


@dataclass(init=False)
class PendingResponse:
    """
    PendingResponse is the response for the pending request.
    It indicates the number of pending records at the user defined source.
    A negative count indicates that the pending information is not available.
    Args:
        count: the number of pending records.
    """

    __slots__ = ("_count",)
    _count: int

    def __init__(self, count: int):
        if not isinstance(count, int):
            raise TypeError(f"Wrong data type: {type(count)} for Pending.count")
        self._count = count

    @property
    def count(self) -> int:
        """Returns the count of pending records"""
        return self._count


@dataclass(init=False)
class PartitionsResponse:
    """
    PartitionsResponse is the response for the partition request.
    It indicates the number of partitions at the user defined source.
    A negative count indicates that the partition information is not available.
    Args:
        count: the number of partitions.
    """

    __slots__ = ("_partitions",)
    _partitions: list[int]

    def __init__(self, partitions: list[int]):
        if not isinstance(partitions, list):
            raise TypeError(f"Wrong data type: {type(partitions)} for Partition.partitions")
        self._partitions = partitions

    @property
    def partitions(self) -> list[int]:
        """Returns the list of partitions"""
        return self._partitions


class Sourcer(metaclass=ABCMeta):
    """
    Provides an interface to write a Sourcer
    which will be exposed over an gRPC server.

    Args:

    """

    def __call__(self, *args, **kwargs):
        """
        Allow to call handler function directly if class instance is sent
        """
        return self.handler(*args, **kwargs)

    @abstractmethod
    async def read_handler(self, datum: ReadRequest, output: NonBlockingIterator):
        """
        Implement this handler function which implements the SourceReadCallable interface.
        read_handler is used to read the data from the source and send the data forward
        for each read request we process num_records and increment the read_idx to indicate that
        the message has been read and the same is added to the ack set
        """
        pass

    @abstractmethod
    def ack_handler(self, ack_request: AckRequest):
        """
        The ack handler is used acknowledge the offsets that have been read, and remove them
        from the to_ack_set
        """
        pass

    @abstractmethod
    def pending_handler(self) -> PendingResponse:
        """
        The simple source always returns zero to indicate there is no pending record.
        """
        pass

    @abstractmethod
    def partitions_handler(self) -> PartitionsResponse:
        """
        The simple source always returns zero to indicate there is no pending record.
        """
        pass


# Create default partition id from the environment variable "NUMAFLOW_REPLICA"
DefaultPartitionId = int(os.getenv("NUMAFLOW_REPLICA", "0"))
SourceReadCallable = Callable[[ReadRequest], Iterable[Message]]
AsyncSourceReadCallable = Callable[[ReadRequest], AsyncIterable[Message]]
SourceAckCallable = Callable[[AckRequest], None]
SourceCallable = Sourcer


def get_default_partitions() -> list[int]:
    """
    Returns the default partition ids.
    """
    return [DefaultPartitionId]



from google.protobuf import empty_pb2 as _empty_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import (
    ClassVar as _ClassVar,
    Iterable as _Iterable,
    Mapping as _Mapping,
    Optional as _Optional,
    Union as _Union,
)

DESCRIPTOR: _descriptor.FileDescriptor

class ReduceRequest(_message.Message):
    __slots__ = ("payload", "operation")

    class WindowOperation(_message.Message):
        __slots__ = ("event", "windows")

        class Event(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
            __slots__ = ()
            OPEN: _ClassVar[ReduceRequest.WindowOperation.Event]
            CLOSE: _ClassVar[ReduceRequest.WindowOperation.Event]
            APPEND: _ClassVar[ReduceRequest.WindowOperation.Event]
        OPEN: ReduceRequest.WindowOperation.Event
        CLOSE: ReduceRequest.WindowOperation.Event
        APPEND: ReduceRequest.WindowOperation.Event
        EVENT_FIELD_NUMBER: _ClassVar[int]
        WINDOWS_FIELD_NUMBER: _ClassVar[int]
        event: ReduceRequest.WindowOperation.Event
        windows: _containers.RepeatedCompositeFieldContainer[Window]
        def __init__(
            self,
            event: _Optional[_Union[ReduceRequest.WindowOperation.Event, str]] = ...,
            windows: _Optional[_Iterable[_Union[Window, _Mapping]]] = ...,
        ) -> None: ...

    class Payload(_message.Message):
        __slots__ = ("keys", "value", "event_time", "watermark", "headers")

        class HeadersEntry(_message.Message):
            __slots__ = ("key", "value")
            KEY_FIELD_NUMBER: _ClassVar[int]
            VALUE_FIELD_NUMBER: _ClassVar[int]
            key: str
            value: str
            def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
        KEYS_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        EVENT_TIME_FIELD_NUMBER: _ClassVar[int]
        WATERMARK_FIELD_NUMBER: _ClassVar[int]
        HEADERS_FIELD_NUMBER: _ClassVar[int]
        keys: _containers.RepeatedScalarFieldContainer[str]
        value: bytes
        event_time: _timestamp_pb2.Timestamp
        watermark: _timestamp_pb2.Timestamp
        headers: _containers.ScalarMap[str, str]
        def __init__(
            self,
            keys: _Optional[_Iterable[str]] = ...,
            value: _Optional[bytes] = ...,
            event_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
            watermark: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
            headers: _Optional[_Mapping[str, str]] = ...,
        ) -> None: ...
    PAYLOAD_FIELD_NUMBER: _ClassVar[int]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    payload: ReduceRequest.Payload
    operation: ReduceRequest.WindowOperation
    def __init__(
        self,
        payload: _Optional[_Union[ReduceRequest.Payload, _Mapping]] = ...,
        operation: _Optional[_Union[ReduceRequest.WindowOperation, _Mapping]] = ...,
    ) -> None: ...

class Window(_message.Message):
    __slots__ = ("start", "end", "slot")
    START_FIELD_NUMBER: _ClassVar[int]
    END_FIELD_NUMBER: _ClassVar[int]
    SLOT_FIELD_NUMBER: _ClassVar[int]
    start: _timestamp_pb2.Timestamp
    end: _timestamp_pb2.Timestamp
    slot: str
    def __init__(
        self,
        start: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        end: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        slot: _Optional[str] = ...,
    ) -> None: ...

class ReduceResponse(_message.Message):
    __slots__ = ("result", "window", "EOF")

    class Result(_message.Message):
        __slots__ = ("keys", "value", "tags")
        KEYS_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        TAGS_FIELD_NUMBER: _ClassVar[int]
        keys: _containers.RepeatedScalarFieldContainer[str]
        value: bytes
        tags: _containers.RepeatedScalarFieldContainer[str]
        def __init__(
            self,
            keys: _Optional[_Iterable[str]] = ...,
            value: _Optional[bytes] = ...,
            tags: _Optional[_Iterable[str]] = ...,
        ) -> None: ...
    RESULT_FIELD_NUMBER: _ClassVar[int]
    WINDOW_FIELD_NUMBER: _ClassVar[int]
    EOF_FIELD_NUMBER: _ClassVar[int]
    result: ReduceResponse.Result
    window: Window
    EOF: bool
    def __init__(
        self,
        result: _Optional[_Union[ReduceResponse.Result, _Mapping]] = ...,
        window: _Optional[_Union[Window, _Mapping]] = ...,
        EOF: bool = ...,
    ) -> None: ...

class ReadyResponse(_message.Message):
    __slots__ = ("ready",)
    READY_FIELD_NUMBER: _ClassVar[int]
    ready: bool
    def __init__(self, ready: bool = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: transform.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x0ftransform.proto\x12\x14sourcetransformer.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1bgoogle/protobuf/empty.proto"\x18\n\tHandshake\x12\x0b\n\x03sot\x18\x01 \x01(\x08"\xbe\x03\n\x16SourceTransformRequest\x12\x45\n\x07request\x18\x01 \x01(\x0b\x32\x34.sourcetransformer.v1.SourceTransformRequest.Request\x12\x37\n\thandshake\x18\x02 \x01(\x0b\x32\x1f.sourcetransformer.v1.HandshakeH\x00\x88\x01\x01\x1a\x95\x02\n\x07Request\x12\x0c\n\x04keys\x18\x01 \x03(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\x12.\n\nevent_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\twatermark\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12R\n\x07headers\x18\x05 \x03(\x0b\x32\x41.sourcetransformer.v1.SourceTransformRequest.Request.HeadersEntry\x12\n\n\x02id\x18\x06 \x01(\t\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x0c\n\n_handshake"\x98\x02\n\x17SourceTransformResponse\x12\x45\n\x07results\x18\x01 \x03(\x0b\x32\x34.sourcetransformer.v1.SourceTransformResponse.Result\x12\n\n\x02id\x18\x02 \x01(\t\x12\x37\n\thandshake\x18\x03 \x01(\x0b\x32\x1f.sourcetransformer.v1.HandshakeH\x00\x88\x01\x01\x1a\x63\n\x06Result\x12\x0c\n\x04keys\x18\x01 \x03(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\x12.\n\nevent_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04tags\x18\x04 \x03(\tB\x0c\n\n_handshake"\x1e\n\rReadyResponse\x12\r\n\x05ready\x18\x01 \x01(\x08\x32\xcf\x01\n\x0fSourceTransform\x12t\n\x11SourceTransformFn\x12,.sourcetransformer.v1.SourceTransformRequest\x1a-.sourcetransformer.v1.SourceTransformResponse(\x01\x30\x01\x12\x46\n\x07IsReady\x12\x16.google.protobuf.Empty\x1a#.sourcetransformer.v1.ReadyResponseb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "transform_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_SOURCETRANSFORMREQUEST_REQUEST_HEADERSENTRY"]._options = None
    _globals["_SOURCETRANSFORMREQUEST_REQUEST_HEADERSENTRY"]._serialized_options = b"8\001"
    _globals["_HANDSHAKE"]._serialized_start = 103
    _globals["_HANDSHAKE"]._serialized_end = 127
    _globals["_SOURCETRANSFORMREQUEST"]._serialized_start = 130
    _globals["_SOURCETRANSFORMREQUEST"]._serialized_end = 576
    _globals["_SOURCETRANSFORMREQUEST_REQUEST"]._serialized_start = 285
    _globals["_SOURCETRANSFORMREQUEST_REQUEST"]._serialized_end = 562
    _globals["_SOURCETRANSFORMREQUEST_REQUEST_HEADERSENTRY"]._serialized_start = 516
    _globals["_SOURCETRANSFORMREQUEST_REQUEST_HEADERSENTRY"]._serialized_end = 562
    _globals["_SOURCETRANSFORMRESPONSE"]._serialized_start = 579
    _globals["_SOURCETRANSFORMRESPONSE"]._serialized_end = 859
    _globals["_SOURCETRANSFORMRESPONSE_RESULT"]._serialized_start = 746
    _globals["_SOURCETRANSFORMRESPONSE_RESULT"]._serialized_end = 845
    _globals["_READYRESPONSE"]._serialized_start = 861
    _globals["_READYRESPONSE"]._serialized_end = 891
    _globals["_SOURCETRANSFORM"]._serialized_start = 894
    _globals["_SOURCETRANSFORM"]._serialized_end = 1101
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: map.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\tmap.proto\x12\x06map.v1\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\xac\x03\n\nMapRequest\x12+\n\x07request\x18\x01 \x01(\x0b\x32\x1a.map.v1.MapRequest.Request\x12\n\n\x02id\x18\x02 \x01(\t\x12)\n\thandshake\x18\x03 \x01(\x0b\x32\x11.map.v1.HandshakeH\x00\x88\x01\x01\x12/\n\x06status\x18\x04 \x01(\x0b\x32\x1a.map.v1.TransmissionStatusH\x01\x88\x01\x01\x1a\xef\x01\n\x07Request\x12\x0c\n\x04keys\x18\x01 \x03(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\x12.\n\nevent_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\twatermark\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x38\n\x07headers\x18\x05 \x03(\x0b\x32\'.map.v1.MapRequest.Request.HeadersEntry\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x0c\n\n_handshakeB\t\n\x07_status"\x18\n\tHandshake\x12\x0b\n\x03sot\x18\x01 \x01(\x08"!\n\x12TransmissionStatus\x12\x0b\n\x03\x65ot\x18\x01 \x01(\x08"\xf0\x01\n\x0bMapResponse\x12+\n\x07results\x18\x01 \x03(\x0b\x32\x1a.map.v1.MapResponse.Result\x12\n\n\x02id\x18\x02 \x01(\t\x12)\n\thandshake\x18\x03 \x01(\x0b\x32\x11.map.v1.HandshakeH\x00\x88\x01\x01\x12/\n\x06status\x18\x04 \x01(\x0b\x32\x1a.map.v1.TransmissionStatusH\x01\x88\x01\x01\x1a\x33\n\x06Result\x12\x0c\n\x04keys\x18\x01 \x03(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\x12\x0c\n\x04tags\x18\x03 \x03(\tB\x0c\n\n_handshakeB\t\n\x07_status"\x1e\n\rReadyResponse\x12\r\n\x05ready\x18\x01 \x01(\x08\x32u\n\x03Map\x12\x34\n\x05MapFn\x12\x12.map.v1.MapRequest\x1a\x13.map.v1.MapResponse(\x01\x30\x01\x12\x38\n\x07IsReady\x12\x16.google.protobuf.Empty\x1a\x15.map.v1.ReadyResponseB7Z5github.com/numaproj/numaflow-go/pkg/apis/proto/map/v1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "map_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"Z5github.com/numaproj/numaflow-go/pkg/apis/proto/map/v1"
    _globals["_MAPREQUEST_REQUEST_HEADERSENTRY"]._options = None
    _globals["_MAPREQUEST_REQUEST_HEADERSENTRY"]._serialized_options = b"8\001"
    _globals["_MAPREQUEST"]._serialized_start = 84
    _globals["_MAPREQUEST"]._serialized_end = 512
    _globals["_MAPREQUEST_REQUEST"]._serialized_start = 248
    _globals["_MAPREQUEST_REQUEST"]._serialized_end = 487
    _globals["_MAPREQUEST_REQUEST_HEADERSENTRY"]._serialized_start = 441
    _globals["_MAPREQUEST_REQUEST_HEADERSENTRY"]._serialized_end = 487
    _globals["_HANDSHAKE"]._serialized_start = 514
    _globals["_HANDSHAKE"]._serialized_end = 538
    _globals["_TRANSMISSIONSTATUS"]._serialized_start = 540
    _globals["_TRANSMISSIONSTATUS"]._serialized_end = 573
    _globals["_MAPRESPONSE"]._serialized_start = 576
    _globals["_MAPRESPONSE"]._serialized_end = 816
    _globals["_MAPRESPONSE_RESULT"]._serialized_start = 740
    _globals["_MAPRESPONSE_RESULT"]._serialized_end = 791
    _globals["_READYRESPONSE"]._serialized_start = 818
    _globals["_READYRESPONSE"]._serialized_end = 848
    _globals["_MAP"]._serialized_start = 850
    _globals["_MAP"]._serialized_end = 967
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: source.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x0csource.proto\x12\tsource.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1bgoogle/protobuf/empty.proto"\x18\n\tHandshake\x12\x0b\n\x03sot\x18\x01 \x01(\x08"\xb1\x01\n\x0bReadRequest\x12/\n\x07request\x18\x01 \x01(\x0b\x32\x1e.source.v1.ReadRequest.Request\x12,\n\thandshake\x18\x02 \x01(\x0b\x32\x14.source.v1.HandshakeH\x00\x88\x01\x01\x1a\x35\n\x07Request\x12\x13\n\x0bnum_records\x18\x01 \x01(\x04\x12\x15\n\rtimeout_in_ms\x18\x02 \x01(\rB\x0c\n\n_handshake"\x81\x05\n\x0cReadResponse\x12.\n\x06result\x18\x01 \x01(\x0b\x32\x1e.source.v1.ReadResponse.Result\x12.\n\x06status\x18\x02 \x01(\x0b\x32\x1e.source.v1.ReadResponse.Status\x12,\n\thandshake\x18\x03 \x01(\x0b\x32\x14.source.v1.HandshakeH\x00\x88\x01\x01\x1a\xe8\x01\n\x06Result\x12\x0f\n\x07payload\x18\x01 \x01(\x0c\x12!\n\x06offset\x18\x02 \x01(\x0b\x32\x11.source.v1.Offset\x12.\n\nevent_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04keys\x18\x04 \x03(\t\x12<\n\x07headers\x18\x05 \x03(\x0b\x32+.source.v1.ReadResponse.Result.HeadersEntry\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xe9\x01\n\x06Status\x12\x0b\n\x03\x65ot\x18\x01 \x01(\x08\x12\x31\n\x04\x63ode\x18\x02 \x01(\x0e\x32#.source.v1.ReadResponse.Status.Code\x12\x38\n\x05\x65rror\x18\x03 \x01(\x0e\x32$.source.v1.ReadResponse.Status.ErrorH\x00\x88\x01\x01\x12\x10\n\x03msg\x18\x04 \x01(\tH\x01\x88\x01\x01" \n\x04\x43ode\x12\x0b\n\x07SUCCESS\x10\x00\x12\x0b\n\x07\x46\x41ILURE\x10\x01"\x1f\n\x05\x45rror\x12\x0b\n\x07UNACKED\x10\x00\x12\t\n\x05OTHER\x10\x01\x42\x08\n\x06_errorB\x06\n\x04_msgB\x0c\n\n_handshake"\xa7\x01\n\nAckRequest\x12.\n\x07request\x18\x01 \x01(\x0b\x32\x1d.source.v1.AckRequest.Request\x12,\n\thandshake\x18\x02 \x01(\x0b\x32\x14.source.v1.HandshakeH\x00\x88\x01\x01\x1a-\n\x07Request\x12"\n\x07offsets\x18\x01 \x03(\x0b\x32\x11.source.v1.OffsetB\x0c\n\n_handshake"\xab\x01\n\x0b\x41\x63kResponse\x12-\n\x06result\x18\x01 \x01(\x0b\x32\x1d.source.v1.AckResponse.Result\x12,\n\thandshake\x18\x02 \x01(\x0b\x32\x14.source.v1.HandshakeH\x00\x88\x01\x01\x1a\x31\n\x06Result\x12\'\n\x07success\x18\x01 \x01(\x0b\x32\x16.google.protobuf.EmptyB\x0c\n\n_handshake"\x1e\n\rReadyResponse\x12\r\n\x05ready\x18\x01 \x01(\x08"]\n\x0fPendingResponse\x12\x31\n\x06result\x18\x01 \x01(\x0b\x32!.source.v1.PendingResponse.Result\x1a\x17\n\x06Result\x12\r\n\x05\x63ount\x18\x01 \x01(\x03"h\n\x12PartitionsResponse\x12\x34\n\x06result\x18\x01 \x01(\x0b\x32$.source.v1.PartitionsResponse.Result\x1a\x1c\n\x06Result\x12\x12\n\npartitions\x18\x01 \x03(\x05".\n\x06Offset\x12\x0e\n\x06offset\x18\x01 \x01(\x0c\x12\x14\n\x0cpartition_id\x18\x02 \x01(\x05\x32\xc8\x02\n\x06Source\x12=\n\x06ReadFn\x12\x16.source.v1.ReadRequest\x1a\x17.source.v1.ReadResponse(\x01\x30\x01\x12:\n\x05\x41\x63kFn\x12\x15.source.v1.AckRequest\x1a\x16.source.v1.AckResponse(\x01\x30\x01\x12?\n\tPendingFn\x12\x16.google.protobuf.Empty\x1a\x1a.source.v1.PendingResponse\x12\x45\n\x0cPartitionsFn\x12\x16.google.protobuf.Empty\x1a\x1d.source.v1.PartitionsResponse\x12;\n\x07IsReady\x12\x16.google.protobuf.Empty\x1a\x18.source.v1.ReadyResponseb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "source_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_READRESPONSE_RESULT_HEADERSENTRY"]._options = None
    _globals["_READRESPONSE_RESULT_HEADERSENTRY"]._serialized_options = b"8\001"
    _globals["_HANDSHAKE"]._serialized_start = 89
    _globals["_HANDSHAKE"]._serialized_end = 113
    _globals["_READREQUEST"]._serialized_start = 116
    _globals["_READREQUEST"]._serialized_end = 293
    _globals["_READREQUEST_REQUEST"]._serialized_start = 226
    _globals["_READREQUEST_REQUEST"]._serialized_end = 279
    _globals["_READRESPONSE"]._serialized_start = 296
    _globals["_READRESPONSE"]._serialized_end = 937
    _globals["_READRESPONSE_RESULT"]._serialized_start = 455
    _globals["_READRESPONSE_RESULT"]._serialized_end = 687
    _globals["_READRESPONSE_RESULT_HEADERSENTRY"]._serialized_start = 641
    _globals["_READRESPONSE_RESULT_HEADERSENTRY"]._serialized_end = 687
    _globals["_READRESPONSE_STATUS"]._serialized_start = 690
    _globals["_READRESPONSE_STATUS"]._serialized_end = 923
    _globals["_READRESPONSE_STATUS_CODE"]._serialized_start = 840
    _globals["_READRESPONSE_STATUS_CODE"]._serialized_end = 872
    _globals["_READRESPONSE_STATUS_ERROR"]._serialized_start = 874
    _globals["_READRESPONSE_STATUS_ERROR"]._serialized_end = 905
    _globals["_ACKREQUEST"]._serialized_start = 940
    _globals["_ACKREQUEST"]._serialized_end = 1107
    _globals["_ACKREQUEST_REQUEST"]._serialized_start = 1048
    _globals["_ACKREQUEST_REQUEST"]._serialized_end = 1093
    _globals["_ACKRESPONSE"]._serialized_start = 1110
    _globals["_ACKRESPONSE"]._serialized_end = 1281
    _globals["_ACKRESPONSE_RESULT"]._serialized_start = 1218
    _globals["_ACKRESPONSE_RESULT"]._serialized_end = 1267
    _globals["_READYRESPONSE"]._serialized_start = 1283
    _globals["_READYRESPONSE"]._serialized_end = 1313
    _globals["_PENDINGRESPONSE"]._serialized_start = 1315
    _globals["_PENDINGRESPONSE"]._serialized_end = 1408
    _globals["_PENDINGRESPONSE_RESULT"]._serialized_start = 1385
    _globals["_PENDINGRESPONSE_RESULT"]._serialized_end = 1408
    _globals["_PARTITIONSRESPONSE"]._serialized_start = 1410
    _globals["_PARTITIONSRESPONSE"]._serialized_end = 1514
    _globals["_PARTITIONSRESPONSE_RESULT"]._serialized_start = 1486
    _globals["_PARTITIONSRESPONSE_RESULT"]._serialized_end = 1514
    _globals["_OFFSET"]._serialized_start = 1516
    _globals["_OFFSET"]._serialized_end = 1562
    _globals["_SOURCE"]._serialized_start = 1565
    _globals["_SOURCE"]._serialized_end = 1893
# @@protoc_insertion_point(module_scope)



from queue import Queue

from pynumaflow._constants import STREAM_EOF


class SyncIterator:
    """A Sync Interator backed by a queue"""

    __slots__ = "_queue"

    def __init__(self, size=0):
        self._queue = Queue(maxsize=size)

    def read_iterator(self):
        item = self._queue.get()
        while True:
            if item == STREAM_EOF:
                break
            yield item
            item = self._queue.get()

    def put(self, item):
        self._queue.put(item)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sink.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\nsink.proto\x12\x07sink.v1\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\xa3\x03\n\x0bSinkRequest\x12-\n\x07request\x18\x01 \x01(\x0b\x32\x1c.sink.v1.SinkRequest.Request\x12+\n\x06status\x18\x02 \x01(\x0b\x32\x1b.sink.v1.TransmissionStatus\x12*\n\thandshake\x18\x03 \x01(\x0b\x32\x12.sink.v1.HandshakeH\x00\x88\x01\x01\x1a\xfd\x01\n\x07Request\x12\x0c\n\x04keys\x18\x01 \x03(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\x12.\n\nevent_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\twatermark\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\n\n\x02id\x18\x05 \x01(\t\x12:\n\x07headers\x18\x06 \x03(\x0b\x32).sink.v1.SinkRequest.Request.HeadersEntry\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x0c\n\n_handshake"\x18\n\tHandshake\x12\x0b\n\x03sot\x18\x01 \x01(\x08"\x1e\n\rReadyResponse\x12\r\n\x05ready\x18\x01 \x01(\x08"!\n\x12TransmissionStatus\x12\x0b\n\x03\x65ot\x18\x01 \x01(\x08"\xfc\x01\n\x0cSinkResponse\x12-\n\x07results\x18\x01 \x03(\x0b\x32\x1c.sink.v1.SinkResponse.Result\x12*\n\thandshake\x18\x02 \x01(\x0b\x32\x12.sink.v1.HandshakeH\x00\x88\x01\x01\x12\x30\n\x06status\x18\x03 \x01(\x0b\x32\x1b.sink.v1.TransmissionStatusH\x01\x88\x01\x01\x1a\x46\n\x06Result\x12\n\n\x02id\x18\x01 \x01(\t\x12\x1f\n\x06status\x18\x02 \x01(\x0e\x32\x0f.sink.v1.Status\x12\x0f\n\x07\x65rr_msg\x18\x03 \x01(\tB\x0c\n\n_handshakeB\t\n\x07_status*0\n\x06Status\x12\x0b\n\x07SUCCESS\x10\x00\x12\x0b\n\x07\x46\x41ILURE\x10\x01\x12\x0c\n\x08\x46\x41LLBACK\x10\x02\x32|\n\x04Sink\x12\x39\n\x06SinkFn\x12\x14.sink.v1.SinkRequest\x1a\x15.sink.v1.SinkResponse(\x01\x30\x01\x12\x39\n\x07IsReady\x12\x16.google.protobuf.Empty\x1a\x16.sink.v1.ReadyResponseb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "sink_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_SINKREQUEST_REQUEST_HEADERSENTRY"]._options = None
    _globals["_SINKREQUEST_REQUEST_HEADERSENTRY"]._serialized_options = b"8\001"
    _globals["_STATUS"]._serialized_start = 855
    _globals["_STATUS"]._serialized_end = 903
    _globals["_SINKREQUEST"]._serialized_start = 86
    _globals["_SINKREQUEST"]._serialized_end = 505
    _globals["_SINKREQUEST_REQUEST"]._serialized_start = 238
    _globals["_SINKREQUEST_REQUEST"]._serialized_end = 491
    _globals["_SINKREQUEST_REQUEST_HEADERSENTRY"]._serialized_start = 445
    _globals["_SINKREQUEST_REQUEST_HEADERSENTRY"]._serialized_end = 491
    _globals["_HANDSHAKE"]._serialized_start = 507
    _globals["_HANDSHAKE"]._serialized_end = 531
    _globals["_READYRESPONSE"]._serialized_start = 533
    _globals["_READYRESPONSE"]._serialized_end = 563
    _globals["_TRANSMISSIONSTATUS"]._serialized_start = 565
    _globals["_TRANSMISSIONSTATUS"]._serialized_end = 598
    _globals["_SINKRESPONSE"]._serialized_start = 601
    _globals["_SINKRESPONSE"]._serialized_end = 853
    _globals["_SINKRESPONSE_RESULT"]._serialized_start = 758
    _globals["_SINKRESPONSE_RESULT"]._serialized_end = 828
    _globals["_SINK"]._serialized_start = 905
    _globals["_SINK"]._serialized_end = 1029
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: reduce.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x0creduce.proto\x12\treduce.v1\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\x98\x04\n\rReduceRequest\x12\x31\n\x07payload\x18\x01 \x01(\x0b\x32 .reduce.v1.ReduceRequest.Payload\x12;\n\toperation\x18\x02 \x01(\x0b\x32(.reduce.v1.ReduceRequest.WindowOperation\x1a\x9e\x01\n\x0fWindowOperation\x12=\n\x05\x65vent\x18\x01 \x01(\x0e\x32..reduce.v1.ReduceRequest.WindowOperation.Event\x12"\n\x07windows\x18\x02 \x03(\x0b\x32\x11.reduce.v1.Window"(\n\x05\x45vent\x12\x08\n\x04OPEN\x10\x00\x12\t\n\x05\x43LOSE\x10\x01\x12\n\n\x06\x41PPEND\x10\x04\x1a\xf5\x01\n\x07Payload\x12\x0c\n\x04keys\x18\x01 \x03(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\x12.\n\nevent_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\twatermark\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12>\n\x07headers\x18\x05 \x03(\x0b\x32-.reduce.v1.ReduceRequest.Payload.HeadersEntry\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"j\n\x06Window\x12)\n\x05start\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x03\x65nd\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04slot\x18\x03 \x01(\t"\xa7\x01\n\x0eReduceResponse\x12\x30\n\x06result\x18\x01 \x01(\x0b\x32 .reduce.v1.ReduceResponse.Result\x12!\n\x06window\x18\x02 \x01(\x0b\x32\x11.reduce.v1.Window\x12\x0b\n\x03\x45OF\x18\x03 \x01(\x08\x1a\x33\n\x06Result\x12\x0c\n\x04keys\x18\x01 \x03(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\x12\x0c\n\x04tags\x18\x03 \x03(\t"\x1e\n\rReadyResponse\x12\r\n\x05ready\x18\x01 \x01(\x08\x32\x8a\x01\n\x06Reduce\x12\x43\n\x08ReduceFn\x12\x18.reduce.v1.ReduceRequest\x1a\x19.reduce.v1.ReduceResponse(\x01\x30\x01\x12;\n\x07IsReady\x12\x16.google.protobuf.Empty\x1a\x18.reduce.v1.ReadyResponseb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "reduce_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_REDUCEREQUEST_PAYLOAD_HEADERSENTRY"]._options = None
    _globals["_REDUCEREQUEST_PAYLOAD_HEADERSENTRY"]._serialized_options = b"8\001"
    _globals["_REDUCEREQUEST"]._serialized_start = 90
    _globals["_REDUCEREQUEST"]._serialized_end = 626
    _globals["_REDUCEREQUEST_WINDOWOPERATION"]._serialized_start = 220
    _globals["_REDUCEREQUEST_WINDOWOPERATION"]._serialized_end = 378
    _globals["_REDUCEREQUEST_WINDOWOPERATION_EVENT"]._serialized_start = 338
    _globals["_REDUCEREQUEST_WINDOWOPERATION_EVENT"]._serialized_end = 378
    _globals["_REDUCEREQUEST_PAYLOAD"]._serialized_start = 381
    _globals["_REDUCEREQUEST_PAYLOAD"]._serialized_end = 626
    _globals["_REDUCEREQUEST_PAYLOAD_HEADERSENTRY"]._serialized_start = 580
    _globals["_REDUCEREQUEST_PAYLOAD_HEADERSENTRY"]._serialized_end = 626
    _globals["_WINDOW"]._serialized_start = 628
    _globals["_WINDOW"]._serialized_end = 734
    _globals["_REDUCERESPONSE"]._serialized_start = 737
    _globals["_REDUCERESPONSE"]._serialized_end = 904
    _globals["_REDUCERESPONSE_RESULT"]._serialized_start = 853
    _globals["_REDUCERESPONSE_RESULT"]._serialized_end = 904
    _globals["_READYRESPONSE"]._serialized_start = 906
    _globals["_READYRESPONSE"]._serialized_end = 936
    _globals["_REDUCE"]._serialized_start = 939
    _globals["_REDUCE"]._serialized_end = 1077
# @@protoc_insertion_point(module_scope)
